package com.css.simulation.resource.server.infra.db.mysql.mapper;


import api.common.pojo.param.model.ConfigPageParam;
import api.common.pojo.po.model.ConfigPO;
import api.common.pojo.po.model.ConfigSensorPO;
import api.common.pojo.vo.model.ConfigSensorVO;
import api.common.pojo.vo.model.ConfigVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

@Mapper
@Repository
public interface ConfigMapper {


    List<ConfigVO> getConfigPageList(ConfigPageParam pageParam);

    ConfigVO getConfigInfo(ConfigVO configVO);

    List<ConfigSensorVO> getConfigSensors(ConfigVO configVO);

    List<ConfigVO> checkConfigName(ConfigPO configPO);

    void insertConfig(ConfigPO configPO);

    void updateConfig(ConfigPO configPO);

    void delConfigSensors(ConfigPO configPO);

    void insertConfigSensors(List<ConfigSensorPO> configSensorPOs);

    int delConfigById(ConfigPO po);

    int getConfigTotal(@Param("createUserId") String currentUserId);
}
